<?php
/**
 * @file
 * The Drush remake extension for Drush.
 */

/**
 * Implements hook_drush_command().
 */
function drush_remake_drush_command() {
  $items = array();

  $items['remake'] = array(
    'description' => "Re-makes an installation profile using drush make",
    'arguments' => array(
      'installation profile' => dt('The name of the installation profile to re-make'),
    ),
    'options' => array(
      'no-working-copy' => dt('Pass this option in to do the remake without the --working-copy option being passed to drush make'),
    ),
    'bootstrap' => DRUSH_BOOTSTRAP_DRUPAL_ROOT, // Bootstrap into a Drupal root.
    'allow-additional-options' => array(
      'make',
    ),
  );

  return $items;
}

/**
 * Validation hook for the drush_remake command.
 */
function drush_drush_remake_remake_validate() {
  $args = func_get_args();
  if (empty($args)) {
    // Try to offer a choice.
    $options = drush_remake_find_profiles();
    // If we can't offer any choices, return an error.
    if (count($options) == 0) {
      return drush_set_error('DRUSH_REMAKE_INCORRECT_ARGUMENTS', dt("You must specify the installation profile to rebuild."));
    }
    // If there's only once choice then just select it.
    elseif (count($options) == 1) {
      reset($options);
      $profile = key($options);
    }
    // Otherwise give the user a choice.
    else {
      $choice = drush_choice($options, dt('Choose an installation profile to remake:'));
      if (!empty($choice)) {
        $profile = $choice;
      }
      else {
        return drush_set_error('DRUSH_REMAKE_INCORRECT_ARGUMENTS', dt("You must specify the installation profile to rebuild."));
      }
    }
  }
  else {
    // Check that the first argument is actually a valid install profile with a make file.
    $profile = $args[0];
  }


  if (!file_exists(DRUPAL_ROOT . '/profiles/' . $profile)) {
    return drush_set_error('DRUSH_REMAKE_UNKNOWN_PROFILE', dt("Could not find an installation profile."));
  }


  if (!file_exists(DRUPAL_ROOT . '/profiles/' . $profile . '/' . $profile . '.make')) {
    return drush_set_error('DRUSH_REMAKE_NO_MAKE_FILE', dt("Could not find a valid make file."));
  }

  drush_set_context('remake_file', $profile);
}

/**
 * Get installation profiles with a make file.
 */
function drush_remake_find_profiles() {
  $options = array();
  foreach (glob(DRUPAL_ROOT . '/profiles/*') as $profile) {
    if (is_dir($profile)) {
      $basename = basename($profile);
      if (file_exists(DRUPAL_ROOT . '/profiles/' . $basename . '/' . $basename . '.make')) {
        $options[$basename] = $basename;
      }
    }
  }
  return $options;
}

/**
 * The main drush remake command callback.
 */
function drush_drush_remake_remake() {
  $profile = drush_get_context('remake_file');
  $directory = DRUPAL_ROOT . '/profiles/' . $profile;

  drush_log(dt('Remaking @profile from @makefile.', array('@profile' => $profile, '@makefile' => $directory . '/' . $profile . '.make')), 'ok');

  // Build the command.
  $args = array();
  $options = drush_redispatch_get_options();
  unset($options['no-working-copy']);

  $args[] = $directory . '/' . $profile . '.make';
  $args[] = '.';

  // We cd to run the command, so the contrib destination should be set to: '.'
  $options['contrib-destination'] = '.';

  // People should use drush make to make with a valid Drupal core.
  $options['no-core'] = TRUE;

  // Set the working-copy option by default, but allow people to not send it.
  if (drush_get_option('no-working-copy', FALSE)) {
    $options['working-copy'] = FALSE;
  }
  else {
    $options['working-copy'] = TRUE;
  }

  // Don't run with a Drupal root.
  $options['root'] = NULL;

  // Call the command
  $backend_options = array(
    'interactive' => TRUE,
  );

  $running_path = getcwd();
  // Convert windows paths.
  $running_path = _drush_convert_path($running_path);

  // Set to the path we want to use.
  chdir($directory);

  drush_invoke_process('@none', 'make', $args, $options, $backend_options);

  // Set back to the previous running path.
  chdir($running_path);
}
